from typing import Optional, Union

from pydantic import (
    BaseModel,
    Field,
    HttpUrl,
    PositiveInt,
    validate_arguments,
    validator,
)

from gpn_clients.utils.patterns import singleton

DEFAULT_HTTP_PORT = 443


@singleton
class NSIConfig(BaseModel):
    HOST: HttpUrl = Field(
        default="https://test-nsi-host.com",
        description="NSI API host",
        alias="host",
    )
    PORT: PositiveInt = Field(
        default=DEFAULT_HTTP_PORT,
        description="NSI API port",
        alias="port",
    )
    API_VERSION: str = Field(
        default="v1",
        alias="api_version",
        description="NSI API version",
        regex="v[0-9]+",
    )

    @classmethod
    @validator("host")
    def drop_last_slash(cls, host: str) -> str:
        if host.endswith("/"):
            return host[:-1]
        return host

    @validate_arguments
    def set_config(
        self,
        host: Optional[Union[HttpUrl, str]] = None,
        port: Optional[Union[PositiveInt, int]] = None,
        api_version: Optional[str] = None,
    ) -> None:
        self.HOST = host or self.HOST
        self.PORT = port or self.PORT
        self.API_VERSION = api_version or self.API_VERSION


nsi_config: NSIConfig = NSIConfig()



from typing import Literal

from pydantic import HttpUrl, PositiveInt, constr, validate_arguments

from gpn_clients.core.config import NSIConfig, nsi_config
from gpn_clients.models.base_nsi_api import AbstractModelsML
from gpn_clients.utils.urls import URLGenerator


class NSIModelsML(AbstractModelsML):
    path: str = "Models"

    def __init__(self, config: NSIConfig = nsi_config) -> None:
        self._config = config
        self._url_generator = URLGenerator(
            host=config.HOST,
            port=config.PORT,
        )

    def get_all(self) -> HttpUrl:
        return self._url_generator.get_full_url(self.path)

    @validate_arguments
    def get_by_id(self, id_: PositiveInt) -> HttpUrl:
        return self._url_generator.get_full_url(self.path, str(id_))

    @validate_arguments
    def get_binary_view(self, id_: PositiveInt) -> HttpUrl:
        binary_view_path: str = "binary"
        return self._url_generator.get_full_url(self.path, str(id_), binary_view_path)

    @validate_arguments
    def get_info_for_name(self, model_name: constr(min_length=1)) -> HttpUrl:
        model_path: str = "info/name"
        get_params = {"name": model_name}
        return self._url_generator.get_full_url_with_get_params(
            self.path,
            model_path,
            get_params=get_params,
        )

    @validate_arguments
    def get_ml_data_for_tag(
            self,
            tag_name: constr(min_length=1),
            input_function: Literal["predict", "train", "calc"],
    ) -> HttpUrl:
        ml_data_path: str = "data/name"
        get_params = {"name": tag_name}
        return self._url_generator.get_full_url_with_get_params(
            self.path,
            ml_data_path,
            input_function,
            get_params=get_params,
        )



from abc import ABC, abstractmethod
from posixpath import join as urljoin
from typing import Dict
from urllib.parse import urlencode

from pydantic import HttpUrl, PositiveInt, parse_obj_as, validate_arguments


class AbstractURLGenerator(ABC):
    @abstractmethod
    def get_full_url(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_base_url(self) -> HttpUrl:
        pass

    @abstractmethod
    def get_full_url_with_get_params(self, *args, **kwargs) -> HttpUrl:
        pass


class URLGenerator(AbstractURLGenerator):
    @validate_arguments
    def __init__(self, host: HttpUrl, port: PositiveInt):
        self._host = host
        self._port = port

    @validate_arguments
    def get_full_url(self, path: str, *additional_paths) -> HttpUrl:
        url = urljoin(self.get_base_url(), path, *additional_paths)
        return parse_obj_as(HttpUrl, url)

    def get_base_url(self) -> HttpUrl:
        return parse_obj_as(HttpUrl, f"{self._host}:{self._port}")

    @validate_arguments
    def get_full_url_with_get_params(
            self,
            *additional_paths,
            get_params: Dict,
    ) -> HttpUrl:
        url_get_params = urlencode(get_params)
        url = urljoin(self.get_base_url(), *additional_paths)
        return parse_obj_as(HttpUrl, f"{url}?{url_get_params}")



from pydantic import HttpUrl, PositiveInt, validate_arguments

from gpn_clients.core.config import NSIConfig, nsi_config
from gpn_clients.models.base_nsi_api import AbstractConfigs
from gpn_clients.utils.urls import URLGenerator


class NSIConfigs(AbstractConfigs):
    path: str = "Configs"

    def __init__(self, config: NSIConfig = nsi_config) -> None:
        self._config = config
        self._url_generator = URLGenerator(
            host=config.HOST,
            port=config.PORT,
        )

    def get_all(self) -> HttpUrl:
        return self._url_generator.get_full_url(self.path)

    @validate_arguments
    def get_by_id(self, id_: PositiveInt) -> HttpUrl:
        return self._url_generator.get_full_url(self.path, str(id_))



from datetime import datetime
from typing import Optional

from pydantic import HttpUrl, conint, constr, validate_arguments

from gpn_clients.core.config import NSIConfig, nsi_config
from gpn_clients.models.base_nsi_api import AbstractTimeSeries
from gpn_clients.utils.urls import URLGenerator


class NSITimeSeries(AbstractTimeSeries):
    path: str = "TimeSeries"

    def __init__(self, config: NSIConfig = nsi_config) -> None:
        self._config = config
        self._url_generator = URLGenerator(
            host=config.HOST,
            port=config.PORT,
        )

    @validate_arguments
    def get_for_timerange(
            self,
            tag_name: constr(min_length=1),
            *,
            start: datetime,
            finish: Optional[datetime] = None,
    ) -> HttpUrl:
        timerange_path: str = "raw/tag/range"
        finish = finish or datetime.utcnow()
        get_params = {
            "tag": tag_name,
            "from": start.isoformat(),
            "to": finish.isoformat(),
        }
        return self._url_generator.get_full_url_with_get_params(
            self.path,
            timerange_path,
            get_params=get_params,
        )

    @validate_arguments
    def get_for_tail(self, tag_name: constr(min_length=1), tail_size: conint(ge=1)) -> HttpUrl:
        tail_path: str = "raw/tag/points"
        get_params = {
            "tag": tag_name,
        }
        return self._url_generator.get_full_url_with_get_params(
            self.path,
            tail_path,
            str(tail_size),
            get_params=get_params,
        )



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Optional

from pydantic import HttpUrl, PositiveInt


class AbstractAlgorithms(ABC):
    @abstractmethod
    def get_all(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_by_id(self, *args, **kwargs) -> HttpUrl:
        pass


class AbstractConfigs(ABC):
    @abstractmethod
    def get_all(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_by_id(self, *args, **kwargs) -> HttpUrl:
        pass


class AbstractModelsML(ABC):
    @abstractmethod
    def get_all(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_by_id(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_binary_view(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_info_for_name(self, model_name: str, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_ml_data_for_tag(self, tag_name: str, *args, **kwargs) -> HttpUrl:
        pass


class AbstractTimeSeries(ABC):
    @abstractmethod
    def get_for_timerange(
        self,
        tag_name: str,
        start: datetime,
        finish: Optional[datetime],
    ) -> HttpUrl:
        pass

    @abstractmethod
    def get_for_tail(self, tag_name: str, tail_size: PositiveInt) -> HttpUrl:
        pass


class AbstractIOConfig(ABC):
    @abstractmethod
    def get_all(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_by_id(self, *args, **kwargs) -> HttpUrl:
        pass

    @abstractmethod
    def get_by_id_config(self, *args, **kwargs) -> HttpUrl:
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gpn_clients',
 'gpn_clients.clients.nsi.v1',
 'gpn_clients.core',
 'gpn_clients.models',
 'gpn_clients.utils']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.10.2,<2.0.0', 'setuptools>=65.7.0,<66.0.0']

setup_kwargs = {
    'name': 'gpn-clients',
    'version': '0.1.1',
    'description': 'Клиенты для получения данных из источников внутри ГПН',
    'long_description': '# 🀣 🅖🅟🅝 🅲🅻🅸🅴🅽🆃🆂 🀣\n\n![Code Coverage](https://img.shields.io/badge/Coverage-90%25-green.svg)\n\nГПН Клиенты предоставляющие интерфейс для работы с источниками данных.\n\n## Базовое использование\n\nКонфигурация клиента на примере клиента NSI\n',
    'author': 'Chumakov Mikhail Dmitrievich',
    'author_email': 'i@m-chumakov-dev.ru',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
